c-----------------------------------------------------------------------
      program calmax
c-----------------------------------------------------------------------
c
c --- CALMAX    Version: 7.0.0          Level: 140912              MAIN
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Read CALPUFF dataset 2.1 or later output files (e.g.,
c               CONC.DAT) and retain the maximum over all sources
c               and files for each receptor and specie-level
c
c --- UPDATES:
c
c --- TNG-2.0.0 (140912) to V7.0.0 (140912)
c        1. Update to System 7
c        2. Change CALPUFF dataset TNG-3.0 = 7.0
c           MODIFIED: HD_CFG, HD_PROC, HD_OUT
c
c --- v1.1 (120915) to TNG-2.0.0 (140912)
c        1. Add CALPUFF dataset TNG-3.0
c           MODIFIED: HD_CFG, HD_PROC
c                     MODULES - HEADER
c
c --- v1.0 (120715) to v1.1 (120915)
c        1. Change all file pathname variables from C*132 to C*256
c           and make control file line-input strings C*320
c           MODIFIED: (main), CONTROL
c                     MODULES v1.1 - USERINF
c                     UTILS   v1.1 - GETCMD
c        2. Add switch to process individual sources within a
c           CALPUFF file or process just the total (default)
c           MODIFIED: (main), CONTROL, GETMAX
c                     MODULES v1.1 - USERINF
c        3. Change the binary and calendar-day output files to 
c           time-block periods
c           NEW:      HD_OUT
c           MODIFIED: (main), CONTROL, HD_CFG, HD_PROC, HD_PER, SETPER
c                     COMP, GETMAX, PUTMAX, PERMAX, PUTPER
c                     ALLOC_2, DEALLOC
c                     MODULES v1.1 - USERINF, DATA, MAXDATA
c
c  EXTERNAL ROUTINES: GETCMD, CONTROL, HD_CFG, HD_PROC, HD_QA,
c                     HD_OUT, HD_PER, SETPER, COMP, GETMAX,
c                     PUTMAX, OERMAX, PUTPER,
c                     ALLOC_1, ALLOC_2, ALLOC_3, DEALLOC
c-----------------------------------------------------------------------
c --- Access module data
      use header
      use userinf
      use maxdata

      implicit none

      integer :: ictrl,iolst,ioinp,ioper,iobin
      parameter(ictrl=10, iolst=11, ioper=13, iobin=14, ioinp=20)

      character(len=12)   :: cversion,clevel
      character(len=256)  :: cfname

      integer :: ione,ierr,ifile,file1

      data ione/1/

c --- Identify version and level of CALMAX:
      cversion='7.0.0'
      clevel='140912'

c --- Control file name (default or specified on command line)
      cfname='CALMAX.INP'
      call GETCMD(cfname)

c --- Open and process control file
      open(ictrl,file=cfname,status='old')
      call CONTROL(ictrl,iolst,cversion,clevel)
      close(ictrl)

c --- Create output files (overwrite any existing)
c --- Time-Block-Period maximum file
      write(iolst,*)
      write(iolst,*)'Creating Time-Pd Maximums File: '//TRIM(perfile)
      write(iolst,*)
      write(*,*)
      write(*,*)'Creating Time-Pd Maximums File: '//TRIM(perfile)
      write(*,*)
      open(ioper,file=perfile,status='unknown')
c --- Write header record to Time-Pd maximum file
      write(ioper,'(a32)') 'PERIODMAX.OUT   1.0             '

c --- CALPUFF binary file
      if(lbinout) then
         write(iolst,*)'Creating   CALPUFF File: '//TRIM(binfile)
         write(iolst,*)
         write(*,*)'Creating   CALPUFF File: '//TRIM(binfile)
         write(*,*)
         open(iobin,file=binfile,status='unknown',
     &              form='unformatted')
      endif

c --- Open CALPUFF input files (No Limit in LF95)
      do ifile=1,nfiles
         ioinparray(ifile)=ioinp+ifile
         write(*,*)'Processing CALPUFF File: '//TRIM(inpfile(ifile))
         write(iolst,*)'Processing File: '//TRIM(inpfile(ifile))
         open(ioinparray(ifile),file=inpfile(ifile),status='old',
     &              form='unformatted')
      enddo

c --- Allocate arrays that depend only on the number of
c --- concurrent CALPUFF files, plus 1 output file
      call ALLOC_1(nfiles+1)

c --- Process top of file header for model run configuration
      do ifile=1,nfiles
         call HD_CFG(iolst,ioinparray(ifile),ioper,iobin,ifile,
     &               cversion,clevel,ierr)
         if(ierr.NE.0) stop 'HALTED at call to HD_CFG -- see list file'
      enddo

c --- Allocate remaining arrays for header
      call ALLOC_2(nfiles+1)

c --- Process rest of file header
      do ifile=1,nfiles
         call HD_PROC(ioinparray(ifile),iobin,ifile)
      enddo

c --- Do selective QA on headers
      call HD_QA(iolst)

c --- Configure periods for output files
      call SETPER(iolst)

c --- Write header data to Time-Period maximum file
      call HD_PER(ioper)
c --- Write header data to binary CALPUFF file
      call HD_OUT(iobin,nfiles+1)

c --- Allocate arrays for assembling maxima
      call ALLOC_3(ione)

c --- Process data records for nfiles
      call COMP(iolst,ioper,iobin,ierr)
      if(ierr.NE.0) stop 'HALTED at call to COMP --- see list file'

c --- Write data records to the Time-Period maximum output file
      call PUTPER(ioper)

c --- Close CALPUFF files
      do ifile=1,nfiles
         CLOSE(ioinparray(ifile))
      enddo
      CLOSE(ioper)
      if(lbinout) CLOSE(iobin)

c --- Release space
      call DEALLOC

c --- Release space for filenames
      deallocate (inpfile,ioinparray)
c --- integer arrays for maxdata
      deallocate (maxdatetime)
c --- Real arrays for maxdata
      deallocate (maxdatag,maxdatad,maxdatact)
      deallocate (maxvals)

 
      write(*,*)
      write(*,*)'CALMAX application completed successfully'

      stop
      end

c---------------------------
c --- Add utility file
      include 'utils.for'
c---------------------------


c-----------------------------------------------------------------------
      subroutine control(ictrl,iolst,cversion,clevel)
c-----------------------------------------------------------------------
c
c --- CALMAX    Version: 7.0.0          Level: 120915           CONTROL
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Obtain program control information
c
c --- UPDATES:
c --- v1.0 (120715) to v1.1 (120915)
c        1. Change all file pathname variables from C*132 to C*256
c           and make all line-input strings C*320
c        2. Add switch to process individual sources within a
c           CALPUFF file or process just the total (default)
c        3. Change the calendar-day output file to a time-block period
c           file
c
c --- INPUT:
c               ICTRL - integer   - Unit number of control file
c               IOLST - integer   - Unit number of output list file
c
c --- OUTPUT:
c     Module /USERINF/
c
c --- CONTROL called by: (main)
c --- CONTROL calls:     DELIM, MK_CASE, RM_BLANK
c----------------------------------------------------------------------
c --- Access module data
      use userinf

      implicit none

c --- Declare arguments
      integer :: ictrl,iolst
      character(len=*)  ::  cversion,clevel

c --- Locals
      logical :: lprocess
      integer :: i,ieof,ierr, icase
      integer :: id1,id2,ieq
      integer :: l1,l2,r1,r2, num
      integer :: mtotal
      integer :: start_month,start_day,start_minute
      integer :: end_month,end_day,end_minute
      integer :: delhour,delsecond
      character(len=320)  ::  blank320,aline320
      character(len=5)    ::  ulcase(2)
      character(len=1)    ::  dlim

      data dlim/'!'/
      data ulcase/'UPPER','LOWER'/

c --- Establish initial values (others in module userinf)
      do i=1,320
         blank320(i:i)=' '
      enddo
      lstfile=blank320(1:256)
      lstfile(1:10)='calmax.lst'


c --- Scan control file for the list-file name, and
c --- number of input files (must allocate storage arrays)
c -----------------------------------------------------------
c --- (List file is not opened yet so error-writes will go to
c --- a FORT-file)
      nfiles=0
      ieof=0
      do while (ieof.EQ.0)
         aline320=blank320
         read(ictrl,'(a320)',iostat=ieof) aline320
         call DELIM(dlim,aline320,id1,id2,ieq,lprocess)
         if(lprocess) then
c ---       Set range for variable name on left and value on right
            l1=id1+1
            l2=ieq-1
            r1=ieq+1
            r2=id2-1
            num=r2-r1+1
c ---       Convert to upper case
            call MK_CASE(iolst,'UPPER',aline320,ierr)
            if(ierr.NE.0) stop 'HALTED in CONTROL'
c ---       Interpret subset of inputs
            if(INDEX(aline320(l1:l2),'INPFILE').GT.0) then
               nfiles=nfiles+1
            elseif(INDEX(aline320(l1:l2),'LSTFILE').GT.0) then
               if(num.GT.256) then
                  write(*,*)'Error in CONTROL: file pathname '//
     &                      'exceeds 256 characters' 
                  write(*,*) TRIM(aline320)
                  stop
               endif
               lstfile=blank320(1:256)
               lstfile=aline320(r1:r2)
c ---          Left-justify
               lstfile=ADJUSTL(lstfile)
            elseif(INDEX(aline320(l1:l2),'LCFILES').GT.0) then
               if(INDEX(aline320(r1:r2),'T').GT.0) then
                  LCFILES=.TRUE.
               else
                  LCFILES=.FALSE.
               endif
            endif
         endif
      enddo

c --- Set pointer for ULCASE (1=upper, 2=lower)
      icase=1
      if(lcfiles) icase=2

c --- Open list file and write header
      call MK_CASE(iolst,ulcase(icase),lstfile,ierr)
      if(ierr.NE.0) stop 'HALTED in CONTROL'
      open(iolst,file=lstfile,status='unknown')
      write(iolst,*)
      write(iolst,*)
      write(iolst,*)'                    '//
     &              '      CALMAX MESSAGE SUMMARY'
      write(iolst,*)'                   VERSION:'//cversion//
     &              '   LEVEL:'//clevel
      write(iolst,*)
      write(iolst,*)

c --- Initial QA
c --- Must have 2 or more CALPUFF files
      if(nfiles.LE.1) then
         write(iolst,*)'Fewer than 2 CALPUFF file names are found '//
     &                 'in the control file'
         write(iolst,*)'Make sure that names are assigned to INPFILE '
         stop 'HALTED in CONTROL -- see list file'
      endif

c --- Rewind control file to complete the processing
      REWIND(ictrl)

c --- Allocate arrays for storing/using input file names
      ALLOCATE (inpfile(nfiles),ioinparray(nfiles))

c --- Process control file again for all contents
c ------------------------------------------------
      nfiles=0
      ieof=0
      do while (ieof.EQ.0)
         aline320=blank320
         read(ictrl,'(a320)',iostat=ieof) aline320
         call DELIM(dlim,aline320,id1,id2,ieq,lprocess)
         if(lprocess) then
c ---       Set range for variable name on left and value on right
            l1=id1+1
            l2=ieq-1
            r1=ieq+1
            r2=id2-1
            num=r2-r1+1
c ---       Convert to upper case
            call MK_CASE(iolst,'UPPER',aline320,ierr)
            if(ierr.NE.0) stop 'HALTED in CONTROL -- see list file'
c ---       Check list of expected inputs
c ---       -----------------------------
            if(INDEX(aline320(l1:l2),'INPFILE').GT.0) then
               if(num.GT.256) then
                  write(*,*)'Error in CONTROL: file pathname '//
     &                      'exceeds 256 characters' 
                  write(*,*) TRIM(aline320)
                  stop
               endif
               nfiles=nfiles+1
               inpfile(nfiles)=blank320(1:256)
               inpfile(nfiles)=aline320(r1:r2)
               inpfile(nfiles)=ADJUSTL(inpfile(nfiles))
c ---          Adjust case
               call MK_CASE(iolst,ulcase(icase),inpfile(nfiles),ierr)
               if(ierr.NE.0) stop 'HALTED in CONTROL -- see list file'

            elseif(INDEX(aline320(l1:l2),'PERFILE').GT.0) then
               if(num.GT.256) then
                  write(*,*)'Error in CONTROL: file pathname '//
     &                      'exceeds 256 characters' 
                  write(*,*) TRIM(aline320)
                  stop
               endif
               perfile=blank320(1:256)
               perfile=aline320(r1:r2)
               perfile=ADJUSTL(perfile)
c ---          Adjust case
               call MK_CASE(iolst,ulcase(icase),perfile,ierr)
               if(ierr.NE.0) stop 'HALTED in CONTROL -- see list file'

            elseif(INDEX(aline320(l1:l2),'BINFILE').GT.0) then
               if(num.GT.256) then
                  write(*,*)'Error in CONTROL: file pathname '//
     &                      'exceeds 256 characters' 
                  write(*,*) TRIM(aline320)
                  stop
               endif
               binfile=blank320(1:256)
               binfile=aline320(r1:r2)
               binfile=ADJUSTL(binfile)
c ---          Adjust case
               call MK_CASE(iolst,ulcase(icase),binfile,ierr)
               if(ierr.NE.0) stop 'HALTED in CONTROL -- see list file'

            elseif(INDEX(aline320(l1:l2),'LSTFILE').GT.0) then
c ---          Nothing:  already processed

            elseif(INDEX(aline320(l1:l2),'LCFILES').GT.0) then
c ---          Nothing:  already processed

            else
c ---          Remove blanks and reset pointers for rest
               call RM_BLANK(aline320)
               call DELIM(dlim,aline320,id1,id2,ieq,lprocess)
c ---          Set range for variable name on left and value on right
               l1=id1+1
               l2=ieq-1
               r1=ieq+1
               r2=id2-1

               if(INDEX(aline320(l1:l2),'S_YEAR').GT.0) then
                  read(aline320(r1:r2),'(i)') start_year

               elseif(INDEX(aline320(l1:l2),'S_MONTH').GT.0) then
                  read(aline320(r1:r2),'(i)') start_month

               elseif(INDEX(aline320(l1:l2),'S_DAY').GT.0) then
                  read(aline320(r1:r2),'(i)') start_day

               elseif(INDEX(aline320(l1:l2),'S_TIME').GT.0) then
                  read(aline320(r1:r2),'(i2.2,1x,i2.2,1x,i2.2)')
     &                 start_hour,start_minute,start_second

               elseif(INDEX(aline320(l1:l2),'E_YEAR').GT.0) then
                  read(aline320(r1:r2),'(i)') end_year

               elseif(INDEX(aline320(l1:l2),'E_MONTH').GT.0) then
                  read(aline320(r1:r2),'(i)') end_month

               elseif(INDEX(aline320(l1:l2),'E_DAY').GT.0) then
                  read(aline320(r1:r2),'(i)') end_day

               elseif(INDEX(aline320(l1:l2),'E_TIME').GT.0) then
                  read(aline320(r1:r2),'(i2.2,1x,i2.2,1x,i2.2)')
     &                 end_hour,end_minute,end_second

               elseif(INDEX(aline320(l1:l2),'MTOTAL').GT.0) then
                  read(aline320(r1:r2),'(i)') mtotal
                  if(mtotal.EQ.1) then
                     ltotal=.TRUE.
                  else
                     ltotal=.FALSE.
                  endif

c --- Always make BIN file (set in module userinf)
               elseif(INDEX(aline320(l1:l2),'LBINOUT').GT.0) then
c                  if(INDEX(aline320(r1:r2),'T').GT.0) then
c                     LBINOUT=.TRUE.
c                  else
c                     LBINOUT=.FALSE.
c                  endif

               else
                  write(iolst,*)'Error in CALMAX control file'
                  write(iolst,*)'Assignment variable unknown:'
                  write(iolst,*)TRIM(aline320)
                  stop 'HALTED in CONTROL -- see list file'
               endif
            endif
         endif
      enddo

c --- Convert minutes to seconds and add to user seconds
      start_second=start_second+start_minute*60
      end_second=end_second+end_minute*60

c --- Convert month, day to Julian day
      call CAL_DAY(iolst,'TOJULIAN',start_year,start_month,start_day,
     &             start_juld,ierr)
      if(ierr.NE.0) stop 'HALTED in CONTROL -- See list file'
      call CAL_DAY(iolst,'TOJULIAN',end_year,end_month,end_day,
     &             end_juld,ierr)
      if(ierr.NE.0) stop 'HALTED in CONTROL -- See list file'

c --- Make long date-time integer
      call DATE_HRSEC(iolst,'TOSNGL',start_year,start_juld,start_hour,
     &                start_second,start_datetime,ierr)
      if(ierr.NE.0) stop 'HALTED in CONTROL -- See list file'
      call DATE_HRSEC(iolst,'TOSNGL',end_year,end_juld,end_hour,
     &                end_second,end_datetime,ierr)
      if(ierr.NE.0) stop 'HALTED in CONTROL -- See list file'

c --- QA
c --- Period cannot be zero or negative
      if(start_datetime.GE.end_datetime) then
         write(iolst,*)'CONTROL:  Error in CALMAX control file'
         write(iolst,*)'Period cannot be zero or negative'
         ierr=1
      endif

c --- End it?
      if(ierr.GT.0)then
         stop 'HALTED in CONTROL -- see list file'
      endif

      return
      end

c-----------------------------------------------------------------------
      subroutine hd_cfg(iolst,ioinp,ioper,ioout,k,cversion,clevel,ierr)
c-----------------------------------------------------------------------
c
c --- CALMAX    Version: 7.0.0          Level: 140912            HD_CFG
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Read first section of a CALPUFF output file header to
c               identify model configuration, and transfer corresponding
c               records to CALPUFF output arrays (with modifications)
c               Write comment records to header of output files
c
c --- UPDATES:
c
c --- TNG-2.0.0 (140912) to v7.0.0 (140912)
c        1. Update CALPUFF dataset TNG-3.0  =  7.0
c
c --- v1.1 (120915) to TNG-2.0.0 (140912)
c        1. Add CALPUFF dataset TNG-3.0
c
c --- v1.0 (120715) to v1.1 (120915)
c        1. Change the calendar-day output file to a time-block period
c           file
c        2. Do not adjust requested period to fit period in any model
c           output file
c        3. Store most output file header data rather than write to
c           file so that it can be modified for running-averages later
c
c --- INPUT:
c          IOLST - integer - File unit number for output List file
c          IOINP - integer - File unit number for input CALPUFF file
c          IOPER - integer - File unit number for Time-Period output
c          IOOUT - integer - File unit number for output CALPUFF file
c              k - integer - Sequential number for input CALPUFF file
c       CVERSION - char*12 - CALMAX version
c         CLEVEL - char*12 - CALMAX level (e.g., date)
c
c --- OUTPUT:
c           IERR - integer - Error flag (0 = No Error)
c
c --- HD_CFG called by: (main)
c --- HD_CFG calls:     none
c----------------------------------------------------------------------
c --- Access module data
      use userinf
      use header

      implicit none

c --- Arguments
      integer :: iolst,ioinp,ioper,ioout,k,ierr
      character(len=12)   :: cversion,clevel

c --- Locals
      logical :: lper,lout
      integer :: msource_out
      integer :: i,iok,idum
      integer :: kbyr,kbjul,kbhr,kbsec,krlg,ireport
      character(len=132)  :: comment, blank132

c --- Use variable names as listed in the CALPUFF Users Guide

c --- Initializations
      ierr=0
      idum=0
      do i=1,132
         blank132(i:i)=' '
      enddo
      msource_out=0

c --- Output is based on the first CALPUFF file read
      lper=.FALSE.
      lout=.FALSE.
      if(k.EQ.1) then
         lper=.TRUE.
         if(lbinout) lout=.TRUE.
      endif

      if(lper) then
c ---    Use 8 comment records
         write(ioper,'(a3)') '  8'
c ---    Write the first 3 comment records to Time-Period output
         do i=1,132
            comment(i:i)='-'
         enddo
         write(ioper,'(a132)') comment
         comment='Prepared by CALMAX Version '//cversion//
     &           ' Level '//clevel//blank132(59:132)
         write(ioper,'(a132)') comment
         comment='Maxima from a set of CALPUFF output files;'//
     &           ' information about first is:'//blank132(71:132)
         write(ioper,'(a132)') comment
      endif

c --- Record #1 - File Declaration -- 24 words
      read(ioinp)  dataset(k),dataver(k),datamod(k)
      if(lout) write(ioout) dataset(k),dataver(k),datamod(k)

c --- QA dataset names
      iok=0
      iok=iok+INDEX(dataset(k),'CONC.DAT')
      iok=iok+INDEX(dataset(k),'DFLX.DAT')
      iok=iok+INDEX(dataset(k),'WFLX.DAT')
      iok=iok+INDEX(dataset(k),'TFLX.DAT')
      if(iok.EQ.0) then
         write(iolst,*)'ERROR in HD_CFG -- Invalid dataset: '//
     &                  dataset//dataver
         write(iolst,*)'Requires Dataset version 2.1 and later format'
         ierr=1
         return
      endif

c --- Comment records
      read(ioinp)  ncom(k)

      if(lout) then
c ---    Add 2 new comment records to output
         write(ioout) (ncom(k)+2)
         comment='Processed by CALMAX Version '//cversion//
     &           ' Level '//clevel//blank132(60:132)
         write(ioout) comment
         comment='Maxima from the set of CALPUFF output files is'//
     &           ' obtained'//blank132(56:132)
         write(ioout) comment
      endif

      do i=1,ncom(k)
         comment=blank132
         read(ioinp)  comment
         if(lout) write(ioout) comment
c ---    Write first 3 comment records to Time-Period output
         if(lper .AND. i.LE.3) write(ioper,'(a132)') comment
      enddo

c --- Control variables (record NCOM+3)

      if(dataver(k).EQ.'2.1             ') then
c ---------------------------------------------
         read(ioinp) cmodel(k),ver(k),level(k),ibyr(k),ibjul(k),
     &      ibhr(k),ibsec(k),xbtz(k),irlg(k),iavg(k),nsecdt(k),nxm(k),
     &      nym(k),dxkm(k),dykm(k),nz(k),xorigkm(k),yorigkm(k),
     &      nssta(k),ibcomp(k),iecomp(k),jbcomp(k),jecomp(k),
     &      ibsamp(k),jbsamp(k),iesamp(k),jesamp(k),
     &      meshdn(k),(nsbytype(i,k),i=1,8),
     &      msource(k),ndrec(k),nctrec(k),lsamp(k),
     &      nspout(k),lcompr(k),i2dmet(k),iutmzn(k),feast(k),fnorth(k),
     &      rnlat0(k),relon0(k),xlat1(k),xlat2(k),
     &      pmap(k),utmhem(k),datum(k),daten(k),
     &      clat0(k),clon0(k),clat1(k),clat2(k)
         nsrctype(k)=8
c ---    Rest of source-types are zero;  8 in this file
         do i=nsrctype(k)+1,mxsrctype
            nsbytype(i,k)=0
         enddo
c ---    No receptor groups
         nrgrp(k)=0
c ---    No specie-level units in header
         lspunit(k)=.FALSE.
c ---    Translate real time zone to character
         call UTC_ZONE(iolst,'REAL2CHR',axbtz(k),xbtz(k),idum,ierr)
         if(ierr.NE.0) stop 'ERROR in HD_CFG -- See list file'

      elseif(dataver(k).EQ.'2.2             ') then
c -------------------------------------------------
         read(ioinp) cmodel(k),ver(k),level(k),ibyr(k),ibjul(k),
     &      ibhr(k),ibsec(k),axbtz(k),irlg(k),iavg(k),nsecdt(k),nxm(k),
     &      nym(k),dxkm(k),dykm(k),nz(k),xorigkm(k),yorigkm(k),
     &      nssta(k),ibcomp(k),iecomp(k),jbcomp(k),jecomp(k),
     &      ibsamp(k),jbsamp(k),iesamp(k),jesamp(k),
     &      meshdn(k),(nsbytype(i,k),i=1,8),
     &      msource(k),ndrec(k),nctrec(k),lsamp(k),
     &      nspout(k),lcompr(k),i2dmet(k),iutmzn(k),feast(k),fnorth(k),
     &      rnlat0(k),relon0(k),xlat1(k),xlat2(k),
     &      pmap(k),utmhem(k),datum(k),daten(k),
     &      clat0(k),clon0(k),clat1(k),clat2(k)
         nsrctype(k)=8
c ---    Rest of source-types are zero;  8 in this file
         do i=nsrctype(k)+1,mxsrctype
            nsbytype(i,k)=0
         enddo
c ---    No receptor groups
         nrgrp(k)=0
c ---    Specie-level units are in header
         lspunit(k)=.TRUE.
c ---    Translate character time zone to real
         call UTC_ZONE(iolst,'CHR2REAL',axbtz(k),xbtz(k),idum,ierr)
         if(ierr.NE.0) stop 'ERROR in HD_CFG -- See list file'

      elseif(dataver(k).EQ.'7.0             ' .OR.
     &       dataver(k).EQ.'TNG-3.0         ') then
c ---------------------------------------------
         read(ioinp) cmodel(k),ver(k),level(k),ibyr(k),ibjul(k),
     &      ibhr(k),ibsec(k),axbtz(k),irlg(k),iavg(k),nsecdt(k),nxm(k),
     &      nym(k),dxkm(k),dykm(k),nz(k),xorigkm(k),yorigkm(k),
     &      nssta(k),ibcomp(k),iecomp(k),jbcomp(k),jecomp(k),
     &      ibsamp(k),jbsamp(k),iesamp(k),jesamp(k),
     &      meshdn(k),nsrctype(k),msource(k),
     &      ndrec(k),nrgrp(k),nctrec(k),lsamp(k),
     &      nspout(k),lcompr(k),i2dmet(k),iutmzn(k),feast(k),fnorth(k),
     &      rnlat0(k),relon0(k),xlat1(k),xlat2(k),
     &      pmap(k),utmhem(k),datum(k),daten(k),
     &      clat0(k),clon0(k),clat1(k),clat2(k)
c ---    Number of each type of source in next record (record NCOM+3a)
         read(ioinp) (nsbytype(i,k),i=1,nsrctype(k))
c ---    Rest of source-types are zero
         do i=nsrctype(k)+1,mxsrctype
            nsbytype(i,k)=0
         enddo
c ---    Specie-level units are in header
         lspunit=.TRUE.
c ---    Translate character time zone to real
         call UTC_ZONE(iolst,'CHR2REAL',axbtz(k),xbtz(k),idum,ierr)
         if(ierr.NE.0) stop 'ERROR in HD_CFG -- See list file'

      else
         write(iolst,*)'ERROR in HD_CFG -- Invalid dataset: '//
     &                  dataset//dataver
         write(iolst,*)'Requires Dataset version 2.1 and later format'
         ierr=1
      endif

c --- Derived data
      nsrc(k)=0
      do i=1,mxsrctype
         nsrc(k)=nsrc(k)+nsbytype(i,k)
      enddo
      if(lsamp(k)) then
         ngx(k)=meshdn(k)*(iesamp(k)-ibsamp(k))+1
         ngy(k)=meshdn(k)*(jesamp(k)-jbsamp(k))+1
      else
         ngx(k)=0
         ngy(k)=0
      endif
      ldisc(k)=.FALSE.
      if(ndrec(k).GT.0) ldisc(k)=.TRUE.
      lctsg(k)=.FALSE.
      if(nctrec(k).GT.0) lctsg(k)=.TRUE.

      if(lper) then
c ---    Last comment for Time-Period output
         comment=blank132
         comment(1:52)=cmodel(k)//' Version '//ver(k)//
     &                 ' Level '//level(k)
         write(ioper,'(a132)') comment
         do i=1,132
            comment(i:i)='-'
         enddo
         write(ioper,'(a132)') comment
      endif

      if(lout) then
c ---    Place values into output header array slot
         dataver(nfiles+1) = dataver(k)
         cmodel(nfiles+1)  = cmodel(k)
         ver(nfiles+1)     = ver(k)  
         level(nfiles+1)   = level(k)
         ibyr(nfiles+1)    = ibyr(k)
         ibjul(nfiles+1)   = ibjul(k)
         ibhr(nfiles+1)    = ibhr(k)
         ibsec(nfiles+1)   = ibsec(k)
         xbtz(nfiles+1)    = xbtz(k)
         axbtz(nfiles+1)   = axbtz(k)
         irlg(nfiles+1)    = irlg(k)
         iavg(nfiles+1)    = iavg(k)
         nsecdt(nfiles+1)  = nsecdt(k)
         nxm(nfiles+1)     = nxm(k)
         nym(nfiles+1)     = nym(k)
         dxkm(nfiles+1)    = dxkm(k)
         dykm(nfiles+1)    = dykm(k)
         nz(nfiles+1)      = nz(k)
         xorigkm(nfiles+1) = xorigkm(k)
         yorigkm(nfiles+1) = yorigkm(k)
         nssta(nfiles+1)   = nssta(k)
         ibcomp(nfiles+1)  = ibcomp(k)
         iecomp(nfiles+1)  = iecomp(k)
         jbcomp(nfiles+1)  = jbcomp(k)
         jecomp(nfiles+1)  = jecomp(k)
         ibsamp(nfiles+1)  = ibsamp(k)
         jbsamp(nfiles+1)  = jbsamp(k)
         iesamp(nfiles+1)  = iesamp(k)
         jesamp(nfiles+1)  = jesamp(k)
         meshdn(nfiles+1)  = meshdn(k)
         do i=1,mxsrctype
            nsbytype(i,nfiles+1)= nsbytype(i,k)
         enddo
         msource(nfiles+1) = msource_out
         nsrctype(nfiles+1)= nsrctype(k)
         ndrec(nfiles+1)   = ndrec(k)
         nrgrp(nfiles+1)   = nrgrp(k)
         nctrec(nfiles+1)  = nctrec(k)
         lsamp(nfiles+1)   = lsamp(k)
         nspout(nfiles+1)  = nspout(k)
         lcompr(nfiles+1)  = lcompr(k)
         i2dmet(nfiles+1)  = i2dmet(k)
         iutmzn(nfiles+1)  = iutmzn(k)
         feast(nfiles+1)   = feast(k)
         fnorth(nfiles+1)  = fnorth(k)
         rnlat0(nfiles+1)  = rnlat0(k)
         relon0(nfiles+1)  = relon0(k)
         xlat1(nfiles+1)   = xlat1(k)
         xlat2(nfiles+1)   = xlat2(k)
         pmap(nfiles+1)    = pmap(k)
         utmhem(nfiles+1)  = utmhem(k)
         datum(nfiles+1)   = datum(k)
         daten(nfiles+1)   = daten(k)
         clat0(nfiles+1)   = clat0(k)
         clon0(nfiles+1)   = clon0(k)
         clat1(nfiles+1)   = clat1(k)
         clat2(nfiles+1)   = clat2(k)
         lspunit(nfiles+1) = lspunit(k)
c ---    Derived data
         nsrc(nfiles+1)    = nsrc(k)
         ngx(nfiles+1)     = ngx(k)
         ngy(nfiles+1)     = ngy(k)
         ldisc(nfiles+1)   = ldisc(k)
         lctsg(nfiles+1)   = lctsg(k)
      endif

      return
      end

c-----------------------------------------------------------------------
      subroutine hd_proc(ioinp,ioout,k)
c-----------------------------------------------------------------------
c
c --- CALMAX    Version: 7.0.0          Level: 140912           HD_PROC
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Read remainder of a CALPUFF output file header and 
c               transfer records to output file arrays
c
c --- UPDATES:
c
c --- TNG-2.0.0 (140912) to v7.0.0 (140912)
c        1. Update CALPUFF dataset TNG-3.0  =  7.0
c
c --- v1.1 (120915) to TNG-2.0.0 (140912)
c        1. Add CALPUFF dataset TNG-3.0 (discrete receptor groups)
c
c --- v1.0 (120715) to v1.1 (120915)
c        1. Store most output file header data rather than write to
c           file so that it can be modified for running-averages later
c
c --- INPUT:
c          IOINP - integer - File unit number for input CALPUFF file
c          IOOUT - integer - File unit number for output CALPUFF file
c              k - integer - Sequential number for output CALPUFF file
c
c --- OUTPUT:
c
c --- HD_PROC called by: (main)
c --- HD_PROC calls:     none
c----------------------------------------------------------------------
c --- Access module data
      use header
      use userinf

      implicit none

c --- Arguments
      integer :: ioinp,ioout,k

c --- Locals
      character(len=16) :: units
      integer :: i,j,n,itype

c --- Title (record NCOM+4)
      read(ioinp)  (title(i,k),i=1,3)

c --- Specie/level list (record NCOM+5)
      read(ioinp)  (csout(i,k),i=1,nspout(k))

c --- Units for each specie/level (record NCOM+5a)
      if(lspunit(k)) then
         read(ioinp)  (csunit(i,k),i=1,nspout(k))
      else
c ---    Set units based on dataset
         units='g/m3            '
         if(INDEX(dataset(k),'CONC.DAT').EQ.0)
     &       units='g/m2/s          '
         do i=1,nspout(k)
            csunit(i,k)=units
         enddo
      endif

c --- Non-gridded receptor coordinates (record NCOM+6)
c --- (New in dataset 7.0 / TNG-3.0: htrec, irgrp)
      if(ndrec(k).NE.0) then
         if(nrgrp(k).EQ.0) then
            read(ioinp) (xrec(i,k),i=1,ndrec(k)),
     &                  (yrec(i,k),i=1,ndrec(k)),
     &                  (zrec(i,k),i=1,ndrec(k))
         elseif(nrgrp(k).GE.1) then
c ---       (7.0 / TNG-3.0 and up record NCOM+6, 6a)
            read(ioinp) (xrec(i,k),i=1,ndrec(k)),
     &                  (yrec(i,k),i=1,ndrec(k)),
     &                  (zrec(i,k),i=1,ndrec(k)),
     &                  (htrec(i,k),i=1,ndrec(k)),
     &                  (irgrp(i,k),i=1,ndrec(k))
            read(ioinp) (argrpnam(n,k),n=1,nrgrp(k))
         endif
      endif

c --- Complex terrain receptor coordinates (record NCOM+7)
      if(nctrec(k).NE.0) then
         read(ioinp)  (xctr(i,k),i=1,nctrec(k)),
     &                (yctr(i,k),i=1,nctrec(k)),
     &                (zctr(i,k),i=1,nctrec(k)),
     &                (ihill(i,k),i=1,nctrec(k))
      endif

c --- Source names (records NCOM+8 to NCOM+15)
c --- Loop over source types
      do i=1,mxsrctype
         if(nsbytype(i,k).GT.0) then
            read(ioinp)  itype,(csrcnam(n,i,k),n=1,nsbytype(i,k))
            if(itype.NE.i) then
c ---          Report problem information and quit
               write(*,*)
               write(*,*)
               write(*,*)'  *  Unexpected source type found, or   *'
               write(*,*)'  *  Unexpected source number found.    *'
               write(*,*)'  *  Source type expected: ',i
               write(*,*)'  *  Source type found   : ',itype
               write(*,*)'  *  Source Names Read:                 *'
               write(*,*) (csrcnam(n,i,k),n=1,nsbytype(i,k))
               stop
            endif
         endif
      enddo

c --- Output is based on the first CALPUFF file read
c --------------------------------------------------
      if(k.NE.1) return

      j=nfiles+1

c --- Title (record NCOM+4)
      do i=1,3
         title(i,j)=title(i,k)
      enddo

c --- Specie/level list (record NCOM+5)
      do i=1,nspout(k)
         csout(i,j)=csout(i,k)
      enddo

c --- Units for each specie/level (record NCOM+5a)
      do i=1,nspout(k)
         csunit(i,j)=csunit(i,k)
      enddo

c --- Non-gridded receptor coordinates (record NCOM+6)
      if(ndrec(k).NE.0) then
         if(nrgrp(k).EQ.0) then
            do i=1,ndrec(k)
               xrec(i,j)=xrec(i,k)
               yrec(i,j)=yrec(i,k)
               zrec(i,j)=zrec(i,k)
            enddo
         elseif(nrgrp(k).GE.1) then
c ---       (7.0 / TNG-3.0 and up record NCOM+6, 6a)
            do i=1,ndrec(k)
               xrec(i,j)=xrec(i,k)
               yrec(i,j)=yrec(i,k)
               zrec(i,j)=zrec(i,k)
               htrec(i,j)=htrec(i,k)
               irgrp(i,j)=irgrp(i,k)
            enddo
            do i=1,ndrec(k)
               irgrp(i,j)=irgrp(i,k)
               argrpnam(i,j)=argrpnam(i,k)
            enddo
         endif
      endif

c --- Complex terrain receptor coordinates (record NCOM+7)
      if(nctrec(k).NE.0) then
         do i=1,nctrec(k)
            xctr(i,j)=xctr(i,k)
            yctr(i,j)=yctr(i,k)
            zctr(i,j)=zctr(i,k)
            ihill(i,j)=ihill(i,k)
         enddo
      endif

c --- Source names (records NCOM+8 ...)
c --- Loop over source types
      do i=1,mxsrctype
         if(nsbytype(i,k).GT.0) then
            do n=1,nsbytype(i,k)
               csrcnam(n,i,j)=csrcnam(n,i,k)
            enddo
         endif
      enddo

      return
      end

c-----------------------------------------------------------------------
      subroutine hd_out(ioout,k)
c-----------------------------------------------------------------------
c
c --- CALMAX    Version: 7.0.0          Level: 140912            HD_OUT
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Write remainder of output file CALPUFF header after
c               comment record section
c
c --- UPDATES:
c
c --- TNG-2.0.0 (140912) to v7.0.0 (140912)
c        1. Update CALPUFF dataset TNG-3.0  =  7.0
c
c --- v1.0 (120915) to TNG-2.0.0 (140912)
c        1. Add CALPUFF dataset TNG-3.0 (discrete receptor groups)
c
c --- INPUT:
c          IOOUT - integer - File unit number for output CALPUFF file
c              k - integer - Array index for stored data
c
c --- OUTPUT:
c
c --- HD_OUT called by: (main)
c --- HD_OUT calls:     none
c----------------------------------------------------------------------
c --- Access module data
      use header

      implicit none

c --- Declare arguments
      integer :: ioout,k

c --- Locals
      logical :: lout
      integer :: i,n,itype

c --- Control variables (record NCOM+3)
      if(dataver(k).EQ.'2.1             ') then
         write(ioout) cmodel(k),ver(k),level(k),ibyr(k),ibjul(k),
     &      ibhr(k),ibsec(k),xbtz(k),irlg(k),iavg(k),nsecdt(k),nxm(k),
     &      nym(k),dxkm(k),dykm(k),nz(k),xorigkm(k),yorigkm(k),
     &      nssta(k),ibcomp(k),iecomp(k),jbcomp(k),jecomp(k),
     &      ibsamp(k),jbsamp(k),iesamp(k),jesamp(k),meshdn(k),
     &      (nsbytype(i,k),i=1,8),msource(k),ndrec(k),nctrec(k),
     &      lsamp(k),nspout(k),lcompr(k),i2dmet(k),iutmzn(k),
     &      feast(k),fnorth(k),rnlat0(k),relon0(k),
     &      xlat1(k),xlat2(k),pmap(k),utmhem(k),datum(k),daten(k),
     &      clat0(k),clon0(k),clat1(k),clat2(k)
      elseif(dataver(k).EQ.'2.2             ') then
         write(ioout) cmodel(k),ver(k),level(k),ibyr(k),ibjul(k),
     &      ibhr(k),ibsec(k),axbtz(k),irlg(k),iavg(k),nsecdt(k),nxm(k),
     &      nym(k),dxkm(k),dykm(k),nz(k),xorigkm(k),yorigkm(k),
     &      nssta(k),ibcomp(k),iecomp(k),jbcomp(k),jecomp(k),
     &      ibsamp(k),jbsamp(k),iesamp(k),jesamp(k),meshdn(k),
     &      (nsbytype(i,k),i=1,8),msource(k),ndrec(k),nctrec(k),
     &      lsamp(k),nspout(k),lcompr(k),i2dmet(k),iutmzn(k),
     &      feast(k),fnorth(k),rnlat0(k),relon0(k),
     &      xlat1(k),xlat2(k),pmap(k),utmhem(k),datum(k),daten(k),
     &      clat0(k),clon0(k),clat1(k),clat2(k)
      elseif(dataver(k).EQ.'7.0             ' .OR.
     &       dataver(k).EQ.'TNG-3.0         ') then
         write(ioout) cmodel(k),ver(k),level(k),ibyr(k),ibjul(k),
     &      ibhr(k),ibsec(k),axbtz(k),irlg(k),iavg(k),nsecdt(k),nxm(k),
     &      nym(k),dxkm(k),dykm(k),nz(k),xorigkm(k),yorigkm(k),
     &      nssta(k),ibcomp(k),iecomp(k),jbcomp(k),jecomp(k),
     &      ibsamp(k),jbsamp(k),iesamp(k),jesamp(k),meshdn(k),
     &      nsrctype(k),msource(k),ndrec(k),nrgrp(k),nctrec(k),
     &      lsamp(k),nspout(k),lcompr(k),i2dmet(k),iutmzn(k),
     &      feast(k),fnorth(k),rnlat0(k),relon0(k),
     &      xlat1(k),xlat2(k),pmap(k),utmhem(k),datum(k),daten(k),
     &      clat0(k),clon0(k),clat1(k),clat2(k)
c ---    Control variables (record NCOM+3a)
         write(ioout) (nsbytype(i,k),i=1,nsrctype(k))
      endif

c --- Title (record NCOM+4)
      write(ioout) (title(i,k),i=1,3)

c --- Specie/level list (record NCOM+5)
      write(ioout) (csout(i,k),i=1,nspout(k))

c --- Units for each specie/level (record NCOM+5a)
      if(lspunit(k)) then
         write(ioout) (csunit(i,k),i=1,nspout(k))
      endif

c --- Non-gridded receptor coordinates (record NCOM+6)
c --- (New in dataset 7.0 / TNG-3.0: htrec, irgrp)
      if(ndrec(k).NE.0) then
         if(nrgrp(k).EQ.0) then
            write(ioout)(xrec(i,k),i=1,ndrec(k)),
     &                  (yrec(i,k),i=1,ndrec(k)),
     &                  (zrec(i,k),i=1,ndrec(k))
         elseif(nrgrp(k).GE.1) then
c ---       (7.0 / TNG-3.0 and up record NCOM+6, 6a)
            write(ioout)(xrec(i,k),i=1,ndrec(k)),
     &                  (yrec(i,k),i=1,ndrec(k)),
     &                  (zrec(i,k),i=1,ndrec(k)),
     &                  (htrec(i,k),i=1,ndrec(k)),
     &                  (irgrp(i,k),i=1,ndrec(k))
            write(ioout)(argrpnam(n,k),n=1,nrgrp(k))
         endif
      endif

c --- Complex terrain receptor coordinates (record NCOM+7)
      if(nctrec(k).NE.0) then
         write(ioout) (xctr(i,k),i=1,nctrec(k)),
     &                (yctr(i,k),i=1,nctrec(k)),
     &                (zctr(i,k),i=1,nctrec(k)),
     &                (ihill(i,k),i=1,nctrec(k))
      endif

c --- Source names (records NCOM+8 ...)
c --- Loop over source types
      do i=1,mxsrctype
         if(nsbytype(i,k).GT.0) then
            write(ioout) i,(csrcnam(n,i,k),n=1,nsbytype(i,k))
         endif
      enddo

      return
      end

c-----------------------------------------------------------------------
      subroutine hd_qa(iolst)
c-----------------------------------------------------------------------
c
c --- CALMAX    Version: 7.0.0          Level: 120715             HD_QA
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Compare selected file header data that should be the
c               same in all files
c
c --- INPUT:
c          IOLST - integer - File unit number for output List file
c
c --- OUTPUT:
c
c --- HD_QA called by: (main)
c --- HD_QA calls:     DATE_HRSEC
c----------------------------------------------------------------------
c --- Access module data
      use header
      use userinf

      implicit none

c --- Declare arguments
      integer :: iolst

c --- Locals
      integer (kind=8) :: start1,startk
      integer :: i,k,nwarn,nfatal,ierr,no

c --- Initializations
      ierr=0
      nwarn=0
      nfatal=0

c --- Notice to list file
      write(iolst,*)
      write(iolst,*)'HD_QA:  Comparing selected header information'
      write(iolst,*)'---------------------------------------------'
      write(iolst,*)

c --- Selected operations on the first file information
      call DATE_HRSEC(iolst,'TOSNGL',ibyr(1),ibjul(1),ibhr(1),ibsec(1),
     &                start1,ierr)
      if(ierr.NE.0) stop 'HALTED in HD_QA -- See list file'

c --- Compare each file header with the first
      do k=2,nfiles
         write(iolst,*) '          File 1 Compared to File ',k

c ---    Dataset type

         if(dataset(1).NE.dataset(k)) then
            nfatal=nfatal+1
            write(iolst,*)'Type of data in file ---'
            write(iolst,*)'  FATAL: dataset ',dataset(1),dataset(k)
         endif

c ---    Times

         call DATE_HRSEC(iolst,'TOSNGL',ibyr(k),ibjul(k),ibhr(k),
     &                   ibsec(k),startk,ierr)
         if(ierr.NE.0) stop 'HALTED in HD_QA -- See list file'

         if(startk.NE.start1 .OR.
     &      irlg(1).NE.irlg(k)) then
            nwarn=nwarn+1
            write(iolst,*)'Times ---'
            write(iolst,*)'WARNING: start  ',start1,startk
            write(iolst,*)'         irlg   ',irlg(1),irlg(k)
         endif

         if(axbtz(1).NE.axbtz(k) .OR.
     &      iavg(1).NE.iavg(k)   .OR.
     &      nsecdt(1).NE.nsecdt(k)) then
            nfatal=nfatal+1
            write(iolst,*)'Times ---'
            write(iolst,*)'  FATAL: axbtz  ',axbtz(1),axbtz(k)
            write(iolst,*)'         iavg   ',iavg(1),iavg(k)
            write(iolst,*)'         nsecdt ',nsecdt(1),nsecdt(k)
         endif


c ---    Receptors

         if(lsamp(1).NEQV.lsamp(k) .OR.
     &      ndrec(1).NE.ndrec(k) .OR.
     &      nctrec(1).NE.nctrec(k)) then
            nfatal=nfatal+1
            write(iolst,*)'Receptor types ---'
            write(iolst,*)'  FATAL: lsamp  ',lsamp(1),lsamp(k)
            write(iolst,*)'         ndrec  ',ndrec(1),ndrec(k)
            write(iolst,*)'         nctrec ',nctrec(1),nctrec(k)
         else

            if(lsamp(1)) then
               if(nxm(1).NE.nxm(k) .OR. nym(1).NE.nym(k) .OR.
     &            dxkm(1).NE.dxkm(k) .OR. dykm(1).NE.dykm(k) .OR.
     &            xorigkm(1).NE.xorigkm(k) .OR.
     &            yorigkm(1).NE.yorigkm(k) .OR.
     &            ibsamp(1).NE.ibsamp(k) .OR.
     &            iesamp(1).NE.iesamp(k) .OR.
     &            jbsamp(1).NE.jbsamp(k) .OR.
     &            jesamp(1).NE.jesamp(k) .OR.
     &            meshdn(1).NE.meshdn(k)) then
                  nfatal=nfatal+1
            write(iolst,*)'Gridded Receptors ---'
            write(iolst,*)'  FATAL: nx     ',nxm(1),nxm(k)
            write(iolst,*)'         ny     ',nym(1),nym(k)
            write(iolst,*)'         dx     ',dxkm(1),dxkm(k)
            write(iolst,*)'         dy     ',dykm(1),dykm(k)
            write(iolst,*)'         xorig  ',xorigkm(1),xorigkm(k)
            write(iolst,*)'         yorig  ',yorigkm(1),yorigkm(k)
            write(iolst,*)'         ibsamp ',ibsamp(k),ibsamp(k)
            write(iolst,*)'         iesamp ',iesamp(k),iesamp(k)
            write(iolst,*)'         jbsamp ',jbsamp(k),jbsamp(k)
            write(iolst,*)'         jesamp ',jesamp(k),jesamp(k)
            write(iolst,*)'         meshdn ',meshdn(1),meshdn(k)
               endif
            endif

            if(ldisc(1)) then
               no=0
               do i=1,ndrec(1)
                  if(xrec(i,1).NE.xrec(i,k) .OR. 
     &               yrec(i,1).NE.yrec(i,k)) no=no+1
               enddo
               if(no.NE.0) then
                  nfatal=nfatal+1
            write(iolst,*)'Discrete Receptors ---'
            write(iolst,*)'  FATAL: number that do not match = ',no
               endif
            endif

            if(lctsg(1)) then
               no=0
               do i=1,nctrec(1)
                  if(xctr(i,1).NE.xctr(i,k) .OR. 
     &               yctr(i,1).NE.yctr(i,k)) no=no+1
               enddo
               if(no.NE.0) then
                  nfatal=nfatal+1
            write(iolst,*)'CTSG Receptors ---'
            write(iolst,*)'  FATAL: number that do not match = ',no
               endif
            endif

         endif


c ---    Species
         if(nspout(1).NE.nspout(k) .OR.
     &      lspunit(1).NEQV.lspunit(k)) then
            write(iolst,*)'Species ---'
            write(iolst,*)'  FATAL: nspout ',nspout(1),nspout(k)
            write(iolst,*)'        lspunit ',lspunit(1),lspunit(k)
         else
            no=0
            do i=1,nspout(1)
               if(csout(i,1).NE.csout(i,k)) no=no+1
            enddo
            if(no.NE.0) then
            write(iolst,*)'Species Output ---'
            write(iolst,*)'  FATAL: number that do not match = ',no
               do i=1,nspout(1)
            write(iolst,*)'        csout   ',csout(i,1),csout(i,k)
               enddo
            endif
         endif

      enddo

      write(iolst,*)'---------------------------------------------'
      write(iolst,*)
      if(nwarn.GT.0) then
         write(*,*)'HD_QA:  File Header WARNINGS -- See list file'
      endif
      if(nfatal.GT.0) then
         stop 'HALTED in HD_QA -- See list file'
      endif

      return

c --- Other items that could be checked:
c ---       lcompr(k),iutmzn(k),feast(k),fnorth(k),
c ---       pmap(k),utmhem(k),datum(k),
c ---       clat0(k),clon0(k),clat1(k),clat2(k)

      end

c-----------------------------------------------------------------------
      subroutine hd_per(ioper)
c-----------------------------------------------------------------------
c
c --- CALMAX    Version: 7.0.0          Level: 120915            HD_PER
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Write remainder of file header
c
c --- UPDATES:
c --- v1.0 (120715) to v1.1 (120915)
c        1. Change the calendar-day output file to a time-block period
c           file
c
c --- INPUT:
c          IOINP - integer - File unit number for input CALPUFF file
c          IOOUT - integer - File unit number for output CALPUFF file
c              k - integer - Sequential number for output CALPUFF file
c
c --- OUTPUT:
c
c --- HD_PER called by: (main)
c --- HD_PER calls:     none
c----------------------------------------------------------------------
c --- Access module data
      use header
      use userinf

      implicit none

c --- Declare arguments
      integer :: ioper

c --- Locals
      character(len=1) :: q
      integer :: i,k

c --- Output is based on the first CALPUFF file read
      k=1

c --- Set single quote
      q=ACHAR(39)


c --- Define date range
      write(ioper,'(a22,i20)') 'START(YYYYJJJHHSSSS) =',start_datetime
      write(ioper,'(a22,i20)') 'END  (YYYYJJJHHSSSS) =',end_datetime
      write(ioper,'(a22,i6)')  'NUMBER_PERIODS       =',nperiods

c --- Define specie-levels and units
      write(ioper,'(a22,i6)') 'NUMBER_SPECIES       =',nspout(k)
      do i=1,nspout(k)
         write(ioper,'(a1,a15,a1,4x,a1,a16,a1)')
     &         q,csout(i,k),q,q,csunit(i,k),q
      enddo


      return
      end

c-----------------------------------------------------------------------
      subroutine setper(iolst)
c-----------------------------------------------------------------------
c
c --- CALMAX    Version: 7.0.0          Level: 120915            SETPER
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Set timing information (control-file, model information)
c               for start-time and runlength in header of output file
c
c --- UPDATES:
c --- v1.0 (120715) to v1.1 (120915)
c        1. Remove constraint that the modeled period cover the period
c           requested in the control file, but use modeled averaging
c           period for output
c        2. Remove constraint that modeled periods align with output
c           periods
c        3. Read first data block in all files to obtain true avg.
c           time (needed for running averages) and reset output file
c           header variables as needed
c
c --- INPUT:
c          IOLST - integer - Unit number of output list file
c
c --- OUTPUT:
c
c --- SETPER called by: (main)
c --- SETPER calls:     GETPER, TIME_DIFF, DATE_HRSEC
c----------------------------------------------------------------------
c --- Access module data
      use header
      use inpdata
      use maxdata
      use userinf

      implicit none

c --- Declare arguments
      integer :: iolst

c --- Locals
      integer         :: k,izero,ierr,ieof
      integer         :: ndimx,ndimy,ndimd,ndimct
      integer         :: ntdiff,iavgsec,krlg
      integer         :: delday,delhour,delsecond
      integer         :: kbyr,kbjul,kbhr,kbsec
c --- Initialization
      ierr=0
      izero=0

c --- Array allocation using k=1 should have resulted in exact match to
c --- shape actually used for all number of receptors --- verify this
      do k=1,nfiles
         if(lsamp(k)) then
            ndimx=SIZE(datag,1)
            ndimy=SIZE(datag,2)
            if(ndimx.NE.ngx(k) .OR. ndimy.NE.ngy(k)) then
               write(iolst,*)'ERROR in SETPER -- array allocation '//
     &                       'does not match gridded receptors in file'
               write(iolst,*)'   Allocated size: ',ndimx,ndimy
               write(iolst,*)'   Size in file  : ',ngx(k),ngy(k)
               stop 'HALTED in SETPER -- see list file'
            endif
         endif
         if(ldisc(k)) then
            ndimd=SIZE(datad,1)
            if(ndimd.NE.ndrec(k)) then
               write(iolst,*)'ERROR in SETPER -- array allocation '//
     &                       'does not match discrete receptors in file'
               write(iolst,*)'   Allocated size: ',ndimd
               write(iolst,*)'   Size in file  : ',ndrec(k)
               stop 'HALTED in SETPER -- see list file'
            endif
         endif
         if(lctsg(k)) then
            ndimct=SIZE(datact,1)
            if(ndimct.NE.nctrec(k)) then
               write(iolst,*)'ERROR in SETPER -- array allocation '//
     &                       'does not match CTSG receptors in file'
               write(iolst,*)'   Allocated size: ',ndimct
               write(iolst,*)'   Size in file  : ',nctrec(k)
               stop 'HALTED in SETPER -- see list file'
            endif
         endif
      enddo

c --- Loop over input files to process first data block
      do k=1,nfiles
         call GETPER(iolst,ioinparray(k),k,ierr,ieof)
         if(ierr.NE.0) stop 'HALTED in SETPER -- see list file'
         if(ieof.NE.0) then
            write(iolst,*)'SETPER -- Reached end-of-file in file ',k
            istatus(k)=9
         elseif(start_datetime.GT.datetime_b(k)) then
c ---       Start of data is before start of time period
            istatus(k)=-1
         else
c ---    Start of data lies within time period or later (use 0 here
c ---    since it will be updated in COMP)
            istatus(k)=0
         endif
c ---    Get averaging time from start-end times
         if(istatus(k).LT.9) then
            call TIME_DIFF(iolst,datetime_b(k),datetime_e(k),
     &                     delday,delhour,delsecond,ierr)
            if(ierr.GT.0) stop 'HALTED in SETPER -- See list file'
            iavgsec=delsecond+3600*(delhour+24*delday)
            if(istepsec.LT.0) then
c ---          First assignment
               istepsec=iavgsec
            else
c ---          Demand subsequent time-periods be the same
               if(istepsec.NE.iavgsec) then
                  write(iolst,*)'SETPER -- Input files cannot have '//
     &                 'different averaging periods'
                  write(iolst,*)'Previous file, time(sec): ',
     &                  k-1,istepsec
                  write(iolst,*)'Current file,  time(sec): ',
     &                  k,iavgsec
                  stop 'HALTED in SETPER -- See list file'
               endif
            endif
         endif
      enddo

c --- Use this time-step to condition the requested period
c --- Difference in requested times (seconds)
      call TIME_DIFF(iolst,start_datetime,end_datetime,
     &               delday,delhour,delsecond,ierr)
      if(ierr.NE.0) stop 'HALTED in SETPER -- See list file'
      ntdiff=(delday*24+delhour)*3600+delsecond

c --- Difference in times as model output steps
      krlg=ntdiff/istepsec

c --- Difference must be multiple of model output step
      if((krlg*istepsec-ntdiff).NE.0) then
         write(iolst,*)'SETPER -- adjusting user end time to '//
     &                 'align with output step'
         write(iolst,*)'   Requested Start  YYYYJJJHHSSSS : ',
     &                     start_datetime
         write(iolst,*)'   Requested End    YYYYJJJHHSSSS : ',
     &                     end_datetime
         write(iolst,*)'   Output Step  SSSS: ',istepsec
         write(iolst,*)'   Time Diff    SSSS: ',ntdiff
         write(iolst,*)'   Number of steps  : ',krlg
c ---    Convert start to individual variables
         call DATE_HRSEC(iolst,'TOINDV',kbyr,kbjul,kbhr,kbsec,
     &                   start_datetime,ierr)
         if(ierr.NE.0) stop 'HALTED in SETPER -- See list file'
c ---    Increment to obtain end time
         call TIME_INCR(iolst,kbyr,kbjul,kbhr,kbsec,izero,
     &                  (krlg*istepsec),ierr)
c ---    Convert end to long date-time integer
         call DATE_HRSEC(iolst,'TOSNGL',kbyr,kbjul,kbhr,kbsec,
     &                   end_datetime,ierr)
         if(ierr.NE.0) stop 'HALTED in SETPER -- See list file'
         write(iolst,*)'   Revised End      YYYYJJJHHSSSS : ',
     &                     end_datetime
      endif

c --- Convert long date-time integer for start to individual variables
c --- for output
      call DATE_HRSEC(iolst,'TOINDV',kbyr,kbjul,kbhr,kbsec,
     &                start_datetime,ierr)
      if(ierr.NE.0) stop 'HALTED in SETPER -- See list file'

c --- Report period information to list file (file 1)
      write(iolst,*)
      write(iolst,*)'   Requested  Start  : ',start_datetime
      write(iolst,*)'                End  : ',end_datetime
      write(iolst,*)'   Output Step  SSSS : ',istepsec
      write(iolst,*)'   Number of Periods : ',krlg
      write(iolst,*)

c --- Pass number of periods to common variable
      nperiods=krlg

c --- Update variables for output header
      ibyr(nfiles+1)=kbyr
      ibjul(nfiles+1)=kbjul
      ibhr(nfiles+1)=kbhr
      ibsec(nfiles+1)=kbsec
      iavg(nfiles+1)=istepsec/nsecdt(nfiles+1)
      irlg(nfiles+1)=krlg

      return
      end

c-----------------------------------------------------------------------
      subroutine comp(iolst,ioper,ioout,ierr)
c-----------------------------------------------------------------------
c
c --- CALMAX    Version: 7.0.0          Level: 120915              COMP
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Process data records to obtain the maximum for each
c               species and receptor for each period, and also the
c               daily maximum over all receptors.
c
c --- UPDATES:
c --- v1.0 (120715) to v1.1 (120915)
c        1. Change the calendar-day output file to a time-block period
c           file
c        2. Allow processing periods to be missing at start and end of
c           input files, and select periods with start-times that fall
c           within current time period
c        3. First data block is read in SETPER before COMP is called
c
c --- INPUT:
c          IOLST - integer - File unit number for output List file
c          IOPER - integer - File unit no. for output Time-Pd Max file
c          IOOUT - integer - File unit number for output CALPUFF file
c
c --- OUTPUT:
c           IERR - integer - Error flag (0 = No Error)
c
c --- COMP called by: (main)
c --- COMP calls:     GETPER, GETMAX, PUTMAX, PERMAX
c                     DATE_HRSEC, TIME_INCR
c----------------------------------------------------------------------
c --- Access module data
      use userinf
      use header
      use inpdata
      use maxdata

      implicit none

c --- Declare arguments
      integer :: iolst,ioinp,ioper,ioout,ierr

c --- Locals
      integer :: k, ieof, izero
      integer :: iyr,jul,ihr,isec
      integer (kind=8) :: ibegper,iendper

c --- Initializations
      ierr=0
      ieof=0
      izero=0
c --- Initialize daily maximum array for all species, periods
      maxvals=0.0
c --- Initialize period date-time array
      maxdatetime=0

c --- Initialize maximum arrays for 1 period
      maxdatag=0.0
      maxdatad=0.0
      maxdatact=0.0

c --- Set begin and end of the first time-period
      current_per=1
      ibegper=start_datetime
      call DATE_HRSEC(iolst,'TOINDV',iyr,jul,ihr,isec,ibegper,ierr)
         if(ierr.NE.0) stop 'HALTED in COMP -- See list file'
      call TIME_INCR(iolst,iyr,jul,ihr,isec,izero,istepsec,ierr)
         if(ierr.NE.0) stop 'HALTED in COMP -- See list file'
      call DATE_HRSEC(iolst,'TOSNGL',iyr,jul,ihr,isec,iendper,ierr)
         if(ierr.NE.0) stop 'HALTED in COMP -- See list file'

c --- Skip to and process first period (all files)
      do k=1,nfiles
c ---    Check timestamp of current data
10       if(ibegper.GT.datetime_b(k)) then
c ---       Start of data is before start of time period
            istatus(k)=-1
         elseif(ibegper.LE.datetime_b(k) .AND.
     &          iendper.GT.datetime_b(k)) then
c ---       Start of data lies within time period
            istatus(k)=0
         elseif(iendper.LE.datetime_b(k)) then
c ---       Start of data is after time period
            istatus(k)=1
         endif
c ---    Process maximums for this file this period
         if(istatus(k).EQ.0) call GETMAX(k)
c ---    Look for more data this period
         if(istatus(k).LT.1) then
            call GETPER(iolst,ioinparray(k),k,ierr,ieof)
            if(ieof.NE.0) then
               write(iolst,*)'COMP -- Reached end-of-file in file ',k
               istatus(k)=9
            endif
            if(istatus(k).NE.9) goto 10
         endif
      enddo
      if(lbinout) then
c ---    Write maximums to output file for this period
         call PUTMAX(iolst,ioout,ibegper,iendper,ierr)
         if(ierr.NE.0) return
      endif
c --- Store maximums for this period
      call PERMAX(ibegper,iendper)
c --- RE-initialize maximum arrays for 1 period
      maxdatag=0.0
      maxdatad=0.0
      maxdatact=0.0

c --- Process all remaining periods
      do current_per=2,nperiods
c ---    Update the time period markers
         ibegper=iendper
         call TIME_INCR(iolst,iyr,jul,ihr,isec,izero,istepsec,ierr)
            if(ierr.NE.0) stop 'HALTED in COMP -- See list file'
         call DATE_HRSEC(iolst,'TOSNGL',iyr,jul,ihr,isec,iendper,ierr)
            if(ierr.NE.0) stop 'HALTED in COMP -- See list file'
c ---    Process data
c         write(*,*) 'Processing period: ',ibegper,iendper
         do k=1,nfiles
c ---       Check timestamp of current data
20          if(ibegper.GT.datetime_b(k)) then
c ---          Start of data is before start of time period
c ---          This should not be possible
               stop 'Start of data is before start of time period'
            elseif(ibegper.LE.datetime_b(k) .AND.
     &             iendper.GT.datetime_b(k)) then
c ---          Start of data lies within time period
               istatus(k)=0
            elseif(iendper.LE.datetime_b(k)) then
c ---          Start of data is after time period
               istatus(k)=1
            endif
c ---       Process maximums for this file this period
            if(istatus(k).EQ.0) call GETMAX(k)
c ---       Look for more data this period
            if(istatus(k).LT.1) then
               call GETPER(iolst,ioinparray(k),k,ierr,ieof)
               if(ieof.NE.0) then
                  write(iolst,*)'COMP -- Reached end-of-file in file ',k
                  istatus(k)=9
               endif
               if(istatus(k).NE.9) goto 20
            endif
         enddo
         if(lbinout) then
c ---       Write averages to output file
            call PUTMAX(iolst,ioout,ibegper,iendper,ierr)
            if(ierr.NE.0) return
         endif
c ---    Update maximums for this period
         call PERMAX(ibegper,iendper)
c ---    RE-initialize maximum arrays for 1 period
         maxdatag=0.0
         maxdatad=0.0
         maxdatact=0.0

      enddo

      return
      end

c-----------------------------------------------------------------------
      subroutine getper(iolst,ioinp,k,ierr,ieof)
c-----------------------------------------------------------------------
c
c --- CALMAX    Version: 7.0.0          Level: 120715            GETPER
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Read and store model data records for 1 time period
c
c               Receptor arrays must be dimensioned EXACTLY for the
c               number(s) of receptors in the CALPUFF file!
c
c --- INPUT:
c          IOLST - integer - File unit number for output List file
c          IOINP - integer - File unit number for input CALPUFF file
c              K - integer - Current CALPUFF file index
c
c --- OUTPUT:
c           IERR - integer - Error flag (0 = No Error)
c           IEOF - integer - End of file flag
c
c --- GETPER called by: COMP
c --- GETPER calls:     DATE_HRSEC, CMPRSIO, RDBDAT
c----------------------------------------------------------------------
c --- Access module data
      use header
      use inpdata

      implicit none

c --- Declare arguments
      integer :: iolst,ioinp,k,ierr,ieof

c --- Locals
      integer :: is,nsources,isp
      integer :: nxy,ncmprs,ione

c --- Initializations
      ione=0
      ieof=0
      ierr=0

c --- Source contribution configuration
      if(msource(k).EQ.0) then
c ---    Sum of all sources
         nsources=1
      else
c ---    Individual sources plus total
         nsources=nsrc(k)+1
      endif

c --- Number of gridded receptors
      nxy=ngx(k)*ngy(k)

c --- Source loop
c ---------------
      do is=1,nsources

c ---    Record 1:  Time period
         read(ioinp,end=999)
     &        nyrb(k),njulb(k),nhrb(k),nsecb(k),
     &        nyre(k),njule(k),nhre(k),nsece(k)
         call DATE_HRSEC(iolst,'TOSNGL',
     &                   nyrb(k),njulb(k),nhrb(k),nsecb(k),
     &                   datetime_b(k),ierr)
         call DATE_HRSEC(iolst,'TOSNGL',
     &                   nyre(k),njule(k),nhre(k),nsece(k),
     &                   datetime_e(k),ierr)
         if(ierr.NE.0) stop 'HALTED in GETPER -- See list file'

c ---    Record 2:  Source information
         read(ioinp,end=999)
     &        istype(is,k),isnum(is,k),sname(is,k),
     &        sxkm(is,k),sykm(is,k)

c ---    Output species-level loop
c --------------------------------
         do isp=1,nspout(k)
            if(lsamp(k)) then
               if(lcompr(k)) then
                  read(ioinp) ncmprs
                  call CMPRSIO(iolst,ioinp,'GETR',
     &                         datag(1,1,isp,is,k),nxy,
     &                         workg,ncmprs,cspecg(isp,k),ierr)
                  if(ierr.NE.0) stop 'HALTED in GETPER -- See list file'
               else
                  call RDBDAT(ioinp,cspecg(isp,k),
     &                        datag(1,1,isp,is,k),ngx(k),ngy(k))
               endif
            endif

            if(ldisc(k)) then
               if(lcompr(k))then
                  read(ioinp) ncmprs
                  call CMPRSIO(iolst,ioinp,'GETR',
     &                         datad(1,isp,is,k),ndrec(k),
     &                         workd,ncmprs,cspecd(isp,k),ierr)
                  if(ierr.NE.0) stop 'HALTED in GETPER -- See list file'
               else
                  call RDBDAT(ioinp,cspecd(isp,k),
     &                        datad(1,isp,is,k),ndrec(k),ione)
               endif
            endif

            if(lctsg(k)) then
               if(lcompr(k))then
                  read(ioinp) ncmprs
                  call CMPRSIO(iolst,ioinp,'GETR',
     &                         datact(1,isp,is,k),nctrec(k),
     &                         workct,ncmprs,cspecct(isp,k),ierr)
                  if(ierr.NE.0) stop 'HALTED in GETPER -- See list file'
               else
                  call RDBDAT(ioinp,cspecct(isp,k),
     &                        datact(1,isp,is,k),nctrec(k),ione)
               endif
            endif
         enddo
c --------------------------------
c ---    End output species-level loop

      enddo
c -------------------
c --- End source loop

      return

999   write(*,*)'GETPER:  Reached end of CALPUFF file ',k
      ieof=1
      return

      end

c-----------------------------------------------------------------------
      subroutine getmax(k)
c-----------------------------------------------------------------------
c
c --- CALMAX    Version: 7.0.0          Level: 120915            GETMAX
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Obtain maximum for each species and receptor for 1 time
c               period from file k
c
c --- UPDATES:
c --- v1.0 (120715) to v1.1 (120915)
c        1. Add switch to process individual sources within a
c           CALPUFF file or process just the total (default)
c
c --- INPUT:
c              K - integer - Current CALPUFF file index
c
c --- OUTPUT:
c
c --- GETMAX called by: COMP
c --- GETMAX calls:     none
c----------------------------------------------------------------------
c --- Access module data
      use maxdata
      use inpdata
      use header
      use userinf

      implicit none

c --- Declare arguments
      integer :: k

c --- Locals
      integer :: ns1,nsources,is,isp,i,j

c --- Initializations

c --- Source contribution configuration
      if(msource(k).EQ.0) then
c ---    One source in file
         ns1=1
         nsources=1
      else
c ---    Individual sources plus total
         if(ltotal) then
c ---       Process just the total
            ns1=nsrc(k)+1
            nsources=ns1
         else
c ---      Do not include the total (look for max due to 1 ind. source)
            ns1=1
            nsources=nsrc(k)
         endif
      endif

c ---    Sources
c        -------
         do is=ns1,nsources

c ---       Output species-level
c           --------------------
            do isp=1,nspout(k)

c ---          Receptors
c              ---------
               if(lsamp(k)) then
                  do j=1,ngy(k)
                  do i=1,ngx(k)
                     maxdatag(i,j,isp)=MAX(maxdatag(i,j,isp),
     &                                     datag(i,j,isp,is,k))
                  enddo
                  enddo
               endif
               if(ldisc(k)) then
                  do i=1,ndrec(k)
                     maxdatad(i,isp)=MAX(maxdatad(i,isp),
     &                                   datad(i,isp,is,k))
                  enddo
               endif
               if(lctsg(k)) then
                  do i=1,nctrec(k)
                     maxdatact(i,isp)=MAX(maxdatact(i,isp),
     &                                    datact(i,isp,is,k))
                  enddo
               endif

            enddo
         enddo

      return
      end

c-----------------------------------------------------------------------
      subroutine putmax(iolst,ioout,ibegper,iendper,ierr)
c-----------------------------------------------------------------------
c
c --- CALMAX    Version: 7.0.0          Level: 120915            PUTMAX
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Write maximum data records for 1 time period
c
c               Receptor arrays must be dimensioned EXACTLY for the
c               number(s) of receptors in the CALPUFF file!
c
c --- UPDATES:
c --- v1.0 (120715) to v1.1 (120915)
c        1. Output times are explicitly computed in COMP and times in
c           input files are no longer correct for output
c
c --- INPUT:
c          IOLST - integer - File unit number for output List file
c          IOOUT - integer - File unit number for output CALPUFF file
c        IBEGPER - integer_8 - Date-time at start of time-block period
c        IENDPER - integer_8 - Date-time at end of time-block period
c
c --- OUTPUT:
c           IERR - integer - Error flag (0 = No Error)
c
c --- PUTMAX called by: COMP
c --- PUTMAX calls:     CMPRSIO, WRBDAT, DATE_HRSEC, TIME_INCR
c----------------------------------------------------------------------
c --- Access module data
      use header
      use maxdata
      use inpdata

      implicit none

c --- Declare arguments
      integer :: iolst,ioout,ierr
      integer (kind=8) :: ibegper,iendper

c --- Locals
      integer :: ione,k,is,isp
      integer :: nxy,ncmprs
      integer :: iyrb,julb,ihrb,isecb
      integer :: iyre,jule,ihre,isece

c --- Initializations
      ione=1
c --- Use header info for file 1, source 1
      k=1
      is=1

c --- Number of gridded receptors
      nxy=ngx(k)*ngy(k)

c --- Extract date and time variables
      call DATE_HRSEC(iolst,'TOINDV',iyrb,julb,ihrb,isecb,ibegper,ierr)
      if(ierr.NE.0) stop 'HALTED in PUTMAX -- See list file'
      call DATE_HRSEC(iolst,'TOINDV',iyre,jule,ihre,isece,iendper,ierr)
      if(ierr.NE.0) stop 'HALTED in PUTMAX -- See list file'

c --- Record 1:  Time period
      write(ioout) iyrb,julb,ihrb,isecb,iyre,jule,ihre,isece

c --- Record 2:  Source information
      write(ioout)
     &     istype(is,k),isnum(is,k),sname(is,k),
     &     sxkm(is,k),sykm(is,k)

c --- Output species-level loop
c -----------------------------
      do isp=1,nspout(k)
         if(lsamp(k)) then
            if(lcompr(k)) then
               ncmprs=nxy
               call CMPRSIO(iolst,ioout,'PUTR',
     &                      maxdatag(1,1,isp),nxy,
     &                      workg,ncmprs,cspecg(isp,k),ierr)
               if(ierr.GT.0) return
            else
               call WRBDAT(ioout,cspecg(isp,k),
     &                     maxdatag(1,1,isp),ngx(k),ngy(k))
            endif
         endif

         if(ldisc(k)) then
            if(lcompr(k))then
               ncmprs=ndrec(k)
               call CMPRSIO(iolst,ioout,'PUTR',
     &                      maxdatad(1,isp),ndrec(k),
     &                      workd,ncmprs,cspecd(isp,k),ierr)
               if(ierr.GT.0) return
            else
               call WRBDAT(ioout,cspecd(isp,k),
     &                     maxdatad(1,isp),ndrec(k),ione)
            endif
         endif

         if(lctsg(k)) then
            if(lcompr(k))then
               ncmprs=nctrec(k)
               call CMPRSIO(iolst,ioout,'PUTR',
     &                      maxdatact(1,isp),nctrec(k),
     &                      workd,ncmprs,cspecct(isp,k),ierr)
               if(ierr.GT.0) return
            else
               call WRBDAT(ioout,cspecct(isp,k),
     &                     maxdatact(1,isp),nctrec(k),ione)
            endif
         endif
      enddo

      return
      end

c-----------------------------------------------------------------------
      subroutine permax(ibegper,iendper)
c-----------------------------------------------------------------------
c
c --- CALMAX    Version: 7.0.0          Level: 120915            PERMAX
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Udate maximum for each species for 1 time period
c
c --- UPDATES:
c --- v1.0 (120715) to v1.1 (120915)
c        1. Change the calendar-day output file to a time-block period
c           file
c
c --- INPUT:
c        IBEGPER - integer_8 - Date-time at start of time-block period
c        IENDPER - integer_8 - Date-time at end of time-block period
c
c --- OUTPUT:
c
c --- PERMAX called by: COMP
c --- PERMAX calls:     none
c----------------------------------------------------------------------
c --- Access module data
      use maxdata
      use header
      use inpdata

      implicit none

c --- Declare arguments
      integer (kind=8) :: ibegper,iendper

c --- Locals
      integer :: k,isp,i,j

c --- Initializations
      k=1

c --- Store times for this period
      maxdatetime(1,current_per)=ibegper
      maxdatetime(2,current_per)=iendper

c --- Process maximum for each species

c --- Output species-level
      do isp=1,nspout(k)

c ---    Receptors
         if(lsamp(k)) then
            do j=1,ngy(k)
            do i=1,ngx(k)
               maxvals(isp,current_per)=MAX
     &        (maxvals(isp,current_per),maxdatag(i,j,isp))
            enddo
            enddo
         endif
         if(ldisc(k)) then
            do i=1,ndrec(k)
               maxvals(isp,current_per)=MAX
     &        (maxvals(isp,current_per),maxdatad(i,isp))
            enddo
         endif
         if(lctsg(k)) then
            do i=1,nctrec(k)
               maxvals(isp,current_per)=MAX
     &        (maxvals(isp,current_per),maxdatact(i,isp))
            enddo
         endif

      enddo

      return
      end

c-----------------------------------------------------------------------
      subroutine putper(ioper)
c-----------------------------------------------------------------------
c
c --- CALMAX    Version: 7.0.0          Level: 120915            PUTPER
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Write maximum data records for 1 time period
c
c               Receptor arrays must be dimensioned EXACTLY for the
c               number(s) of receptors in the CALPUFF file!
c
c --- UPDATES:
c --- v1.0 (120715) to v1.1 (120915)
c        1. Change the calendar-day output file to a time-block period
c           file
c
c --- INPUT:
c          IOPER - integer - File unit number for output Time-Pd file
c
c --- OUTPUT:
c
c --- PUTPER called by: (main)
c --- PUTPER calls:     none
c----------------------------------------------------------------------
c --- Access module data
      use maxdata
      use header
      use userinf

      implicit none

c --- Declare arguments
      integer :: ioper

c --- Locals
      integer :: i,isp

c --- Initializations

c --- Write records containing YYYYJJJ, HHMM timestamp for the max
c --- value for each species-level

      do i=1,nperiods
         write(ioper,'(2i14,4x,200e15.7)')
     &         maxdatetime(1,i),maxdatetime(2,i),
     &        (maxvals(isp,i),isp=1,nspout(1))
      enddo

      return
      end

c-----------------------------------------------------------------------
      subroutine alloc_1(k)
c-----------------------------------------------------------------------
c
c --- CALMAX    Version: 7.0.0          Level: 140912           ALLOC_1
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Allocate arrays of header information that depend only
c               on the number of CALPUFF files processed
c
c --- UPDATES:
c
c --- v1.0 (120715) to TNG-2.0.0 (140912)
c        1. Add CALPUFF dataset TNG-3.0 (discrete receptor groups)
c
c --- INPUT:
c              k - integer - Number of CALPUFF files
c
c --- OUTPUT:
c
c --- ALLOC_1 called by: (main)
c --- ALLOC_1 calls:     none
c----------------------------------------------------------------------
c --- Access module data
      use header

      implicit none

c --- Arguments
      integer :: k

c --- Logical arrays (read from file)
      allocate (lcompr(k),lsamp(k))

c --- Logical arrays (derived)
      allocate (ldisc(k),lctsg(k),lspunit(k))

c --- Character arrays (read from file)
      allocate (cmodel(k),ver(k),level(k))
      allocate (dataset(k),dataver(k),datamod(k))
      allocate (title(3,k))
      allocate (pmap(k),datum(k),axbtz(k),daten(k),utmhem(k))
      allocate (clat0(k),clon0(k),clat1(k),clat2(k))

c --- Integer arrays (read from file)
      allocate (ncom(k),ibyr(k),ibjul(k),ibhr(k),ibsec(k),irlg(k))
      allocate (iavg(k),nsecdt(k),nxm(k),nym(k),nz(k),nssta(k))
      allocate (ibcomp(k),iecomp(k),jbcomp(k),jecomp(k))
      allocate (ibsamp(k),jbsamp(k),iesamp(k),jesamp(k))
      allocate (meshdn(k),nsrctype(k),msource(k))
      allocate (ndrec(k),nrgrp(k),nctrec(k),nspout(k))
      allocate (i2dmet(k),iutmzn(k))
      allocate (nsbytype(mxsrctype,k))

c --- Integer arrays (derived)
      allocate (nsrc(k),ngx(k),ngy(k))

c --- Real arrays (read from file)
      allocate (xbtz(k),dxkm(k),dykm(k),xorigkm(k),yorigkm(k))
      allocate (feast(k),fnorth(k))
      allocate (rnlat0(k),relon0(k),xlat1(k),xlat2(k))

      return
      end

c-----------------------------------------------------------------------
      subroutine alloc_2(k)
c-----------------------------------------------------------------------
c
c --- CALMAX    Version: 7.0.0          Level: 140912           ALLOC_2
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Allocate arrays of header information that depend on
c               the number of CALPUFF files processed and other
c               configuration data, and all data arrays
c
c --- UPDATES:
c
c --- v1.1 (120915) to TNG-2.0.0 (140912)
c        1. Add CALPUFF dataset TNG-3.0
c
c --- v1.0 (120715) to v1.1 (120915)
c        1. Add flag array for time-period status
c
c --- INPUT:
c              k - integer - Number of CALPUFF files
c
c --- OUTPUT:
c
c --- ALLOC_2 called by: (main)
c --- ALLOC_2 calls:     none
c----------------------------------------------------------------------
c --- Access module data
      use userinf
      use header
      use inpdata

      implicit none

c --- Arguments
      integer :: k

c --- Locals
      integer :: i,n
      integer :: mxspout,mxsrc,mxsrc1,mxsrc2
      integer :: mxdrec,mxctrec,mxnx,mxny,mxrgrp

c --- Get maximum number of species-levels across files
      mxspout=1
      do i=1,k
         mxspout=MAX(mxspout,nspout(i))
      enddo

c --- Get maximum number of discrete receptors across files
      mxdrec=1
      do i=1,k
         mxdrec=MAX(mxdrec,ndrec(i))
      enddo

c --- Get maximum number of gridded receptors across files
      mxnx=1
      mxny=1
      do i=1,k
         mxnx=MAX(mxnx,ngx(i))
         mxny=MAX(mxny,ngy(i))
      enddo

c --- Get maximum number of CTSG receptors across files
      mxctrec=1
      do i=1,k
         mxctrec=MAX(mxctrec,nctrec(i))
      enddo

c --- Get maximum number of each source type across files
      mxsrc=1
      mxsrc1=1
      mxsrc2=1
      do i=1,k
         mxsrc=MAX(mxsrc,nsrc(i))
         do n=1,mxsrctype
            mxsrc1=MAX(mxsrc1,nsbytype(n,i))
         enddo
         if(msource(i).EQ.1) mxsrc2=MAX(mxsrc2,nsrc(i)+1)
      enddo

c --- Get maximum number of receptor groups across files
      mxrgrp=1
      do i=1,k
         mxrgrp=MAX(mxrgrp,nrgrp(i))
      enddo

c --- Character arrays for header
      allocate (csout(mxspout,k),csunit(mxspout,k))
      allocate (csrcnam(mxsrc1,mxsrctype,k))
      allocate (argrpnam(mxrgrp,k))

c --- Integer arrays for header
      allocate (ihill(mxctrec,k))
      allocate (irgrp(mxdrec,k))
      
c --- Real arrays for header
      allocate (xrec(mxdrec,k),yrec(mxdrec,k))
      allocate (zrec(mxdrec,k),htrec(mxdrec,k))
      allocate (xctr(mxctrec,k),yctr(mxctrec,k),zctr(mxctrec,k))


c --- Character arrays for data
      allocate (sname(mxsrc,k))
      allocate (cspecg(mxspout,k),cspecd(mxspout,k),cspecct(mxspout,k))

c --- Integer arrays for data
      allocate (istype(mxsrc,k),isnum(mxsrc,k))
      allocate (nyrb(k),njulb(k))
      allocate (nhrb(k),nsecb(k))
      allocate (nyre(k),njule(k))
      allocate (nhre(k),nsece(k))
      allocate (datetime_b(k),datetime_e(k))
      allocate (istatus(k))

c --- Real arrays for data
      allocate (sxkm(mxsrc,k),sykm(mxsrc,k))
      allocate (datag(mxnx,mxny,mxspout,mxsrc2,k))
      allocate (datad(mxdrec,mxspout,mxsrc2,k))
      allocate (datact(mxctrec,mxspout,mxsrc2,k))

c --- Real work arrays
      allocate (workg(mxnx,mxny))
      allocate (workd(mxdrec))
      allocate (workct(mxctrec))

      return
      end

c-----------------------------------------------------------------------
      subroutine alloc_3(k)
c-----------------------------------------------------------------------
c
c --- CALMAX    Version: 7.0.0          Level: 120715           ALLOC_3
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Allocate arrays for assembling maxima
c
c --- INPUT:
c              k - integer - Number of the CALPUFF array to use
c
c --- OUTPUT:
c
c --- ALLOC_3 called by: (main)
c --- ALLOC_3 calls:     none
c----------------------------------------------------------------------
c --- Access module data
      use header
      use maxdata
      use userinf

      implicit none
      integer :: k

c --- Real arrays for data

c --- Maximum over all sources
      allocate (maxdatag(ngx(k),ngy(k),nspout(k)))
      allocate (maxdatad(ndrec(k),nspout(k)))
      allocate (maxdatact(nctrec(k),nspout(k)))

c --- Maximum over all sources and receptors for each species and 
c --- period
      allocate (maxvals(nspout(k),nperiods))
      allocate (maxdatetime(2,nperiods))

      return
      end

c-----------------------------------------------------------------------
      subroutine dealloc
c-----------------------------------------------------------------------
c
c --- CALMAX    Version: 7.0.0          Level: 140912           DEALLOC
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  De-Allocate arrays
c
c --- UPDATES:
c
c --- v1.1 (120915) to TNG-2.0.0 (140912)
c        1. Add CALPUFF dataset TNG-3.0

c --- v1.0 (120715) to v1.1 (120915)
c        1. Add flag array for time-period status
c
c --- INPUT:
c
c --- OUTPUT:
c
c --- DEALLOC called by: (main)
c --- DEALLOC calls:     none
c----------------------------------------------------------------------
c --- Access module data
      use header
      use inpdata
      use maxdata

      implicit none


c --- Logical arrays for header
      deallocate (lcompr,lsamp,ldisc,lctsg,lspunit)

c --- Character arrays for header
      deallocate (cmodel,ver,level)
      deallocate (dataset,dataver,datamod)
      deallocate (pmap,datum,axbtz,daten,utmhem)
      deallocate (clat0,clon0,clat1,clat2)
      deallocate (title,csout,csunit)
      deallocate (csrcnam,argrpnam)

c --- Integer arrays for header
      deallocate (ncom,ibyr,ibjul,ibhr,ibsec,irlg)
      deallocate (iavg,nsecdt,nxm,nym,nz,nssta)
      deallocate (ibcomp,iecomp,jbcomp,jecomp)
      deallocate (ibsamp,jbsamp,iesamp,jesamp)
      deallocate (meshdn,msource)
      deallocate (nsrctype,nsbytype,nrgrp,irgrp)
      deallocate (ndrec,nctrec,nspout,i2dmet,iutmzn)
      deallocate (ihill)
      deallocate (nsrc,ngx,ngy)
      
c --- Real arrays for header
      deallocate (xbtz,dxkm,dykm,xorigkm,yorigkm)
      deallocate (feast,fnorth)
      deallocate (rnlat0,relon0,xlat1,xlat2)
      deallocate (xrec,yrec,zrec,htrec)
      deallocate (xctr,yctr,zctr)

c --- Character arrays for inpdata
      deallocate (sname,cspecg,cspecd,cspecct)

c --- Integer arrays for inpdata
      deallocate (istype,isnum)
      deallocate (nyrb,njulb,nhrb,nsecb)
      deallocate (nyre,njule,nhre,nsece)
      deallocate (istatus)

c --- Real arrays for inpdata
      deallocate (sxkm,sykm)
      deallocate (datag,datad,datact)

c --- Real work arrays for inpdata
      deallocate (workg,workd,workct)

      return
      end

